/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Preferences Toolbar 3.
 *
 * The Initial Developer of the Original Code is
 * Manuel Reimer <manuel.reimer@gmx.de>.
 * Portions created by the Initial Developer are Copyright (C) 2___
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s): Manuel Reimer <Manuel.Reimer@gmx.de>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

// +-
// |  the contents of this file will be loaded into the context of the global
// |  nsIAppShellService.hiddenDOMWindow.goPrefBar!
// +-

const prefbarVersion = 20070318;

var InFirefox = false;

var PrefService = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService);
var PrefBranch = PrefService.getBranch("");

// Global Initialization function
// Called once per Mozilla Session
function Init() {
    dump("initializing PrefBar global Object");

    // Check if we are in Firefox.
    InFirefox = CheckForFirefox();

    // Load default preferences Settings if needed
    Include("chrome://prefbar/content/defaultPreferences.js", this);
    SetDefaultPrefs();

    // Load RDF stuff
    this.RDF = new Object;
    Include("chrome://prefbar/content/prefbarRDF.js", this.RDF);
    RDF.Init(this);

    // Load Importer/Exporter
    this.ImpExp = new Object;
    Include("chrome://prefbar/content/importexport.js", this.ImpExp);
    ImpExp.Init(this);

    // Init observers
    var ObserverService = Components.classes["@mozilla.org/observer-service;1"].getService();
    ObserverService = ObserverService.QueryInterface(Components.interfaces.nsIObserverService);
    // Init observer for listening on Profile changes via "Extras ->
    // Change Profile". In this situation we have to "re-initialise" some
    // parts of PrefBar
    ProfChangeObserver.goPrefBar = this;
    ObserverService.addObserver(ProfChangeObserver,"profile-after-change",false);
    // Init observer for resetting the UA string. The observer will be called
    // by mozilla if the suite is about to shutdown. Means if all browser 
    // *and* all mail, composer, chat.... windows have been closed.
    UAResetObserver.goPrefBar = this;
    ObserverService.addObserver(UAResetObserver,"quit-application",false);

    // Init Preference Observers
    var prefs = Components.classes["@mozilla.org/preferences-service;1"]
        .getService(Components.interfaces.nsIPrefBranch);
    var pbi = prefs.QueryInterface(Components.interfaces.nsIPrefBranchInternal);
    pbi.addObserver("extensions.prefbar.show_prefbar_menu", PrefObserver, false);
    pbi.addObserver("extensions.prefbar.display_on", PrefObserver, false);
    pbi.addObserver("extensions.prefbar.slimbuttons", PrefObserver, false);

    // If not in Firefox, then check if we need to fix language settings
    if (!InFirefox) {
        var did_lang_fix = RepairLanguageSetting("prefbar");
        if (did_lang_fix && navigator.preference("extensions.prefbar.information_langfix"))
            alert("A mozilla bug caused PrefBar to be installed with the wrong locale selected.\nPrefBar has corrected this problem.\nYou'll have the toolbar in your locale after the next restart");
    }
}

var ProfChangeObserver = {
    observe: function(subject,topic,data){
        if (data == "switch") {
            // The user has switched the profile. At first we have to do
            // a check if PrefBar is available in the new profile
            var urlprefbar = "chrome://prefbar/content/prefbar.js";
            var ioservice = Components.classes["@mozilla.org/network/io-service;1"]
            .getService(Components.interfaces.nsIIOService);
            try {
                var channel = ioservice.newChannel(urlprefbar, null, null);
                var istream = channel.open();
                istream.close();
            }
            catch (e) {
                goPrefBar.dump("PrefBar: Profile changed, but no PrefBar here...");
                return;
            }

            // Now as we are sure that PrefBar really exists here,
            // reinitialize some modules
            goPrefBar.SetDefaultPrefs();
            goPrefBar.RDF.Init(goPrefBar);
            goPrefBar.ImpExp.Init(goPrefBar);
            goPrefBar.dump("PrefBar: Profile changed, reinitialized modules!");
        }
    }
};

var UAResetObserver = {
    observe: function(subject,topic,data){
	/* Reset the User Agent string pref when exiting the browser completely.

	This is to prevent crashes when using the Java Plugin with the UA set
	to that of IE. See mozilla bug 83376.
	
	As an added bonus prevents you from forgetting that you changed it and
	sending the wrong UA everywhere for three weeks.
	*/

	var cs = unescape("%64%6F%6E%5F%74%5F%72%65%73%65%74%5F%75%61");
	var csp = "extensions.prefbar." + cs;
	if (navigator.preference(csp) == "true") {
	    navigator.preference(csp, "disabled");
	    return;
	}

	var prefBranch = goPrefBar.PrefService.getBranch("");
	try { prefBranch.clearUserPref("general.useragent.override"); } catch(e) {}
        try { prefBranch.clearUserPref("general.appname.override"); } catch(e) {}
        try { prefBranch.clearUserPref("general.appversion.override"); } catch(e) {}
        try { prefBranch.clearUserPref("general.platform.override"); } catch (e) {}
    }
};

// preferences observer
var PrefObserver = {
    observe: function(subject, topic, data) {
        // Notify all open PrefBar instances about the change
        var windowMediator = Components.classes['@mozilla.org/appshell/window-mediator;1'].getService(Components.interfaces.nsIWindowMediator);
	var browserWindows = windowMediator.getEnumerator("navigator:browser");
	while(browserWindows.hasMoreElements())
	{
            var browserWindow = browserWindows.getNext();
            if ("prefbarPrefObserver" in browserWindow) {
                dump("PrefObserver: Notifying browser Window \"" + browserWindow + "\" about: " + data);
                browserWindow.prefbarPrefObserver(subject, topic, data);
            }
	}
    }
};


function CheckForFirefox() {
    var urlbrowser = "chrome://navigator/content/navigator.js";
    var ioservice = Components.classes["@mozilla.org/network/io-service;1"]
	.getService(Components.interfaces.nsIIOService);
    try {
	var channel = ioservice.newChannel(urlbrowser, null, null);
	var istream = channel.open();
	istream.close();
    }
    catch (e) {
	return true;
    }
    return false;
}

function dump(message) {
    var debugpref;
    try {
        debugpref = navigator.preference("extensions.prefbar.debug");
    } catch(e) {
        debugpref = false;
    }

    if (debugpref != undefined && debugpref == true) {
	ConsoleDump(message);
	//window.dump(message + "\n");
    }
}

function ConsoleDump(aText) {
    var csClass = Components.classes['@mozilla.org/consoleservice;1'];
    var cs = csClass.getService(Components.interfaces.nsIConsoleService);
    cs.logStringMessage(aText);
}

function Include(asURL, aoContext) {
    try {
	var oLoader = Components.classes["@mozilla.org/moz/jssubscript-loader;1"]
	    .getService(Components.interfaces.mozIJSSubScriptLoader);
	oLoader.loadSubScript(asURL, aoContext);
	return true;
    }
    catch(e) {
	dump("!ERROR! include of File failed: " + asURL);
	return false;
    }
}

function GetString(filename, stringname) {
    var StringBundle;
    var strBundleService = Components.classes["@mozilla.org/intl/stringbundle;1"].getService(Components.interfaces.nsIStringBundleService);

    StringBundle = strBundleService.createBundle("chrome://prefbar/locale/" + filename); 

    if (StringBundle) {
	return StringBundle.GetStringFromName(stringname);
    }
    return "";
}

// Fix to be able to have more than one Language in one extension.
// Only needed with SeaMonkey. Just pass the name of the extensions as it was
// registered while Installation.
// Feel free to use in your extension, but please leave my name (Manuel Reimer)
// in your credits.
// Function returns "true" if language had to be fixed. "false" otherwise.
function RepairLanguageSetting(extName) {
    //var RDF = Components.classes["@mozilla.org/rdf/rdf-service;1"].getService(Components.interfaces.nsIRDFService);
    var RDF = goPrefBar.RDF.RDFService;
    var rExtSection = RDF.GetResource("urn:mozilla:package:" + extName);
    var rBaseUrl = RDF.GetResource("http://www.mozilla.org/rdf/chrome#baseURL");

    // Get URL to user chrome database
    var dirService = Components.classes["@mozilla.org/file/directory_service;1"].getService(Components.interfaces.nsIProperties);
    var fileprotocolService = Components.classes["@mozilla.org/network/protocol;1?name=file"].getService(Components.interfaces.nsIFileProtocolHandler);
    var fuserchrome = dirService.get("UChrm", Components.interfaces.nsIFile);
    fuserchrome.append("chrome.rdf");
    var fnuserchrome = fileprotocolService.getURLSpecFromFile(fuserchrome);

    // Open database
    var uDS = RDF.GetDataSourceBlocking(fnuserchrome);

    // Check if Extension is installed to this chrome.rdf file
    var lBaseUrl = uDS.GetTarget(rExtSection, rBaseUrl, true);
    if (!lBaseUrl) return false;

    // Get all installed langauges for the extension.
    var arrExtLang = new Array();
    var rLangs = RDF.GetResource("urn:mozilla:locale:root");
    var langcontainer = Components.classes["@mozilla.org/rdf/container;1"].createInstance(Components.interfaces.nsIRDFContainer);
    langcontainer.Init(uDS, rLangs);
    var langelements = langcontainer.GetElements();
    while (langelements.hasMoreElements()) {
        var langelement = langelements.getNext();
        var strlang = langelement.QueryInterface(Components.interfaces.nsIRDFResource).Value;
        var strextlang = strlang + ":" + extName;
        var rextlangSec = RDF.GetResource(strextlang);
        var lelUrl = uDS.GetTarget(rextlangSec, rBaseUrl, true);
        if (lelUrl) {
            // test-load the locale to be sure it is really installed
            var strelUrl = lelUrl.QueryInterface(Components.interfaces.nsIRDFLiteral).Value;
            strelUrl += "contents.rdf";
            var testds;
            try {
                testds = RDF.GetDataSourceBlocking(strelUrl);
            } catch(e) {}
            if (testds) arrExtLang.push(strextlang);
        }
    }

    // Try to find the best matching language
    var strRealExtLang;
    var rlangname = navigator.language;
    var rlangparts = rlangname.split("-");
    for (var index in arrExtLang) {
        var curlang = arrExtLang[index];
        var curlang_parts = curlang.split(":");
        var langname = curlang_parts[curlang_parts.length - 2];
        var langparts = langname.split("-");
        if (langname == navigator.language) {
            strRealExtLang = curlang;
        }
        if (!strRealExtLang && rlangparts[0] == langparts[0]) {
            strRealExtLang = curlang;
        }
    }
    if (!strRealExtLang) return false;

    // Store the new language setting to the database if needed.
    var rExtLang = RDF.GetResource(strRealExtLang);
    var rSelectedLocale = RDF.GetResource("http://www.mozilla.org/rdf/chrome#selectedLocale");
    var rOldExtLang = uDS.GetTarget(rExtSection, rSelectedLocale, true);
    if (!rOldExtLang) return false;
    var strOldExtLang = rOldExtLang.QueryInterface(Components.interfaces.nsIRDFResource).Value;
    if (strOldExtLang != strRealExtLang) {
        uDS.Change(rExtSection, rSelectedLocale, rOldExtLang, rExtLang);

        uDS.QueryInterface(Components.interfaces.nsIRDFRemoteDataSource).Flush();
        return true;
    }
    return false;
}
